# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .actor import Actor
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Messaging(UncheckedBaseModel):
    """
    The Messaging object encompasses information related to instant messaging, such as sending messages, adding a user to a channel, etc.
    """

    channel_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the channel that the event pertains to.
    """

    channel_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the channel that the event pertains to.
    """

    domains: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of domains participating in the chat session.
    """

    has_foreign_tenant_users: typing.Optional[bool] = pydantic.Field(default=None)
    """
    True if the event pertains to a channel or group which contains users from a foreign tenant.
    """

    has_guest_users: typing.Optional[bool] = pydantic.Field(default=None)
    """
    True if the event pertains to a channel or group which contains guest users.
    """

    has_unauthenticated_users: typing.Optional[bool] = pydantic.Field(default=None)
    """
    True if the event pertains to a channel or group which contains users that are not authenticated.
    """

    members: typing.Optional[typing.List[Actor]] = pydantic.Field(default=None)
    """
    Actors to whom the event pertains.
    """

    message_urls: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    URLs contained in the messages which are relevant to the chat session or event.
    """

    session_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the session that the event pertains to, e.g. a chat specific to a meeting, or a thread within a channel.
    """

    session_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the session.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    UID of the event, e.g. unique message ID.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from .types.list_accounts_response import ListAccountsResponse
from ..core.unchecked_base_model import construct_type
from ..common.errors.bad_request_error import BadRequestError
from ..common.types.problem import Problem
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.internal_server_error import InternalServerError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..account_base.types.account_id import AccountId
from .types.get_account_response import GetAccountResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..organization_base.types.environment import Environment
from .types.create_account_response import CreateAccountResponse
from ..organization_base.types.organization_id import OrganizationId
import datetime as dt
from .types.update_account_response import UpdateAccountResponse
from ..common.types.patch_operation import PatchOperation
from .types.patch_account_response import PatchAccountResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AccountsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        total: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListAccountsResponse:
        """
        Returns a list of all `Account` objects. For more information on
        Organizations and Accounts, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of `Account` objects to return in this page. Defaults to 100.

        start_after : typing.Optional[str]
            Return `Account` objects starting after this `name`.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by. Defaults to `name`. To control the direction of the sorting, append
            `[asc]` or `[desc]` to the field name. For example, `name[desc]` will sort the results by `name` in descending order.
            The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the
            ordering is applied in the order the fields are specified.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter.
            If used more than once, the queries are ANDed together.

        total : typing.Optional[bool]
            Return total number of accounts in the system, respecting all applied filters. This is expensive, use sparingly.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListAccountsResponse

        Examples
        --------
        from synqly import SynqlyManagement

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.accounts.list(
            limit=1,
            start_after="string",
            order="string",
            filter="string",
            total=True,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/accounts",
            method="GET",
            params={
                "limit": limit,
                "start_after": start_after,
                "order": order,
                "filter": filter,
                "total": total,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListAccountsResponse,
                    construct_type(
                        type_=ListAccountsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        account_id: AccountId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetAccountResponse:
        """
        Returns the `Account` object matching `{accountId}`. For more information on
        Organizations and Accounts, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        account_id : AccountId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAccountResponse

        Examples
        --------
        from synqly import SynqlyManagement

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.accounts.get(
            account_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/accounts/{jsonable_encoder(account_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetAccountResponse,
                    construct_type(
                        type_=GetAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        name: typing.Optional[str] = OMIT,
        fullname: typing.Optional[str] = OMIT,
        environment: typing.Optional[Environment] = OMIT,
        labels: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateAccountResponse:
        """
        Creates an `Account` object. For more information on Organizations and
        Accounts, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        name : typing.Optional[str]
            Unique short name for this Account (lowercase [a-z0-9_-], can be used in URLs). Also used for case insensitive duplicate name detection and default sort order. Defaults to AccountId if both name and fullname are not specified.

        fullname : typing.Optional[str]
            Human friendly display name for this Account, will auto-generate 'name' field (if 'name' is not specified). Defaults to the same value as the 'name' field if not specified.

        environment : typing.Optional[Environment]
            Environment this account runs in. Defaults to `prod` if not specified.

        labels : typing.Optional[typing.Sequence[str]]
            User defined labels that apply to this account. Labels are limited to 48 characters in length and must contain only letters, numbers, underscores, hyphens, colons, and periods. The label values can be used in role bindings to limit the scope of permissions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateAccountResponse

        Examples
        --------
        from synqly import SynqlyManagement
        from synqly.organization_base import Environment

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.accounts.create(
            name="string",
            fullname="string",
            environment=Environment.TEST,
            labels=["string"],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/accounts",
            method="POST",
            json={
                "name": name,
                "fullname": fullname,
                "environment": environment,
                "labels": labels,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateAccountResponse,
                    construct_type(
                        type_=CreateAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        account_id: AccountId,
        *,
        id: AccountId,
        fullname: str,
        organization_id: OrganizationId,
        environment: Environment,
        name: str,
        created_at: dt.datetime,
        updated_at: dt.datetime,
        labels: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateAccountResponse:
        """
        Updates the `Account` object matching `{accountId}`. For more information on
        Organizations and Accounts, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        account_id : AccountId

        id : AccountId

        fullname : str
            Human friendly display name for this account.

        organization_id : OrganizationId
            Organization that manages this Account.

        environment : Environment
            Environment this account runs in.

        name : str
            Human-readable name for this resource

        created_at : dt.datetime
            Time object was originally created

        updated_at : dt.datetime
            Last time object was updated

        labels : typing.Optional[typing.Sequence[str]]
            User defined labels that apply to this account. These values can be used in role bindings to limit the scope of permissions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateAccountResponse

        Examples
        --------
        import datetime

        from synqly import SynqlyManagement
        from synqly.organization_base import Environment

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.accounts.update(
            account_id="string",
            id="string",
            fullname="string",
            organization_id="string",
            environment=Environment.TEST,
            labels=["string"],
            name="string",
            created_at=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
            updated_at=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/accounts/{jsonable_encoder(account_id)}",
            method="PUT",
            json={
                "id": id,
                "fullname": fullname,
                "organization_id": organization_id,
                "environment": environment,
                "labels": labels,
                "name": name,
                "created_at": created_at,
                "updated_at": updated_at,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UpdateAccountResponse,
                    construct_type(
                        type_=UpdateAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def patch(
        self,
        account_id: AccountId,
        *,
        request: typing.Sequence[PatchOperation],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatchAccountResponse:
        """
        Patches the `Account` object matching `{accountId}`. For more information on
        Organizations and Accounts, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        account_id : AccountId

        request : typing.Sequence[PatchOperation]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatchAccountResponse

        Examples
        --------
        from synqly import SynqlyManagement
        from synqly.common import PatchOp, PatchOperation

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.accounts.patch(
            account_id="string",
            request=[
                PatchOperation(
                    op=PatchOp.ADD,
                    path="string",
                    from_="string",
                    value={"key": "value"},
                )
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/accounts/{jsonable_encoder(account_id)}",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PatchAccountResponse,
                    construct_type(
                        type_=PatchAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        account_id: AccountId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes the `Account` matching `{accountId}`. Deleting an `Account` also deletea
        all `Tokens` and `Credentials` belonging to the `Account`.

        Parameters
        ----------
        account_id : AccountId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from synqly import SynqlyManagement

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.accounts.delete(
            account_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/accounts/{jsonable_encoder(account_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncAccountsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        total: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListAccountsResponse:
        """
        Returns a list of all `Account` objects. For more information on
        Organizations and Accounts, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of `Account` objects to return in this page. Defaults to 100.

        start_after : typing.Optional[str]
            Return `Account` objects starting after this `name`.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by. Defaults to `name`. To control the direction of the sorting, append
            `[asc]` or `[desc]` to the field name. For example, `name[desc]` will sort the results by `name` in descending order.
            The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the
            ordering is applied in the order the fields are specified.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter.
            If used more than once, the queries are ANDed together.

        total : typing.Optional[bool]
            Return total number of accounts in the system, respecting all applied filters. This is expensive, use sparingly.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListAccountsResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyManagement

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.accounts.list(
                limit=1,
                start_after="string",
                order="string",
                filter="string",
                total=True,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/accounts",
            method="GET",
            params={
                "limit": limit,
                "start_after": start_after,
                "order": order,
                "filter": filter,
                "total": total,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListAccountsResponse,
                    construct_type(
                        type_=ListAccountsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        account_id: AccountId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetAccountResponse:
        """
        Returns the `Account` object matching `{accountId}`. For more information on
        Organizations and Accounts, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        account_id : AccountId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAccountResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyManagement

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.accounts.get(
                account_id="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/accounts/{jsonable_encoder(account_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetAccountResponse,
                    construct_type(
                        type_=GetAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        name: typing.Optional[str] = OMIT,
        fullname: typing.Optional[str] = OMIT,
        environment: typing.Optional[Environment] = OMIT,
        labels: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateAccountResponse:
        """
        Creates an `Account` object. For more information on Organizations and
        Accounts, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        name : typing.Optional[str]
            Unique short name for this Account (lowercase [a-z0-9_-], can be used in URLs). Also used for case insensitive duplicate name detection and default sort order. Defaults to AccountId if both name and fullname are not specified.

        fullname : typing.Optional[str]
            Human friendly display name for this Account, will auto-generate 'name' field (if 'name' is not specified). Defaults to the same value as the 'name' field if not specified.

        environment : typing.Optional[Environment]
            Environment this account runs in. Defaults to `prod` if not specified.

        labels : typing.Optional[typing.Sequence[str]]
            User defined labels that apply to this account. Labels are limited to 48 characters in length and must contain only letters, numbers, underscores, hyphens, colons, and periods. The label values can be used in role bindings to limit the scope of permissions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateAccountResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyManagement
        from synqly.organization_base import Environment

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.accounts.create(
                name="string",
                fullname="string",
                environment=Environment.TEST,
                labels=["string"],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/accounts",
            method="POST",
            json={
                "name": name,
                "fullname": fullname,
                "environment": environment,
                "labels": labels,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateAccountResponse,
                    construct_type(
                        type_=CreateAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        account_id: AccountId,
        *,
        id: AccountId,
        fullname: str,
        organization_id: OrganizationId,
        environment: Environment,
        name: str,
        created_at: dt.datetime,
        updated_at: dt.datetime,
        labels: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateAccountResponse:
        """
        Updates the `Account` object matching `{accountId}`. For more information on
        Organizations and Accounts, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        account_id : AccountId

        id : AccountId

        fullname : str
            Human friendly display name for this account.

        organization_id : OrganizationId
            Organization that manages this Account.

        environment : Environment
            Environment this account runs in.

        name : str
            Human-readable name for this resource

        created_at : dt.datetime
            Time object was originally created

        updated_at : dt.datetime
            Last time object was updated

        labels : typing.Optional[typing.Sequence[str]]
            User defined labels that apply to this account. These values can be used in role bindings to limit the scope of permissions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateAccountResponse

        Examples
        --------
        import asyncio
        import datetime

        from synqly import AsyncSynqlyManagement
        from synqly.organization_base import Environment

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.accounts.update(
                account_id="string",
                id="string",
                fullname="string",
                organization_id="string",
                environment=Environment.TEST,
                labels=["string"],
                name="string",
                created_at=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                updated_at=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/accounts/{jsonable_encoder(account_id)}",
            method="PUT",
            json={
                "id": id,
                "fullname": fullname,
                "organization_id": organization_id,
                "environment": environment,
                "labels": labels,
                "name": name,
                "created_at": created_at,
                "updated_at": updated_at,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UpdateAccountResponse,
                    construct_type(
                        type_=UpdateAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def patch(
        self,
        account_id: AccountId,
        *,
        request: typing.Sequence[PatchOperation],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatchAccountResponse:
        """
        Patches the `Account` object matching `{accountId}`. For more information on
        Organizations and Accounts, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        account_id : AccountId

        request : typing.Sequence[PatchOperation]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatchAccountResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyManagement
        from synqly.common import PatchOp, PatchOperation

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.accounts.patch(
                account_id="string",
                request=[
                    PatchOperation(
                        op=PatchOp.ADD,
                        path="string",
                        from_="string",
                        value={"key": "value"},
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/accounts/{jsonable_encoder(account_id)}",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PatchAccountResponse,
                    construct_type(
                        type_=PatchAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        account_id: AccountId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes the `Account` matching `{accountId}`. Deleting an `Account` also deletea
        all `Tokens` and `Credentials` belonging to the `Account`.

        Parameters
        ----------
        account_id : AccountId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyManagement

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.accounts.delete(
                account_id="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/accounts/{jsonable_encoder(account_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Image(UncheckedBaseModel):
    """
    The Image object provides a description of a specific Virtual Machine (VM) or Container image. Defined by D3FEND <a target='_blank' href='https://d3fend.mitre.org/dao/artifact/d3f:ContainerImage/'>d3f:ContainerImage</a>.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The image labels.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The image name. For example: <code>elixir</code>.
    """

    path: typing.Optional[str] = pydantic.Field(default=None)
    """
    The full path to the image file.
    """

    tag: typing.Optional[str] = pydantic.Field(default=None)
    """
    The image tag. For example: <code>1.11-alpine</code>.
    """

    uid: str = pydantic.Field()
    """
    The unique image ID. For example: <code>77af4d6b9913</code>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    BridgeSelector,
    BridgeSelector_Id,
    BridgeSelector_Labels,
    CreateIntegrationRequest,
    CreateIntegrationResponse,
    CreateIntegrationResponseResult,
    GetIntegrationResponse,
    Integration,
    ListAccountIntegrationsResponse,
    ListIntegrationOptions,
    ListIntegrationsResponse,
    PatchIntegrationResponse,
    UpdateIntegrationRequest,
    UpdateIntegrationResponse,
    VerifyIntegrationRequest,
    WebhookConfig,
    WebhookItem,
)

__all__ = [
    "BridgeSelector",
    "BridgeSelector_Id",
    "BridgeSelector_Labels",
    "CreateIntegrationRequest",
    "CreateIntegrationResponse",
    "CreateIntegrationResponseResult",
    "GetIntegrationResponse",
    "Integration",
    "ListAccountIntegrationsResponse",
    "ListIntegrationOptions",
    "ListIntegrationsResponse",
    "PatchIntegrationResponse",
    "UpdateIntegrationRequest",
    "UpdateIntegrationResponse",
    "VerifyIntegrationRequest",
    "WebhookConfig",
    "WebhookItem",
]



# This file was auto-generated by Fern from our API Definition.

from . import common, indicator, open_vocab
from .common import (
    CommonProperties,
    ExternalReference,
    GranularMarking,
    Id,
    Identity,
    KillChainPhase,
    MarkingDefinition,
)
from .indicator import Indicator
from .open_vocab import IdentityClassOv, IndicatorTypeOv, PatternTypeOv

__all__ = [
    "CommonProperties",
    "ExternalReference",
    "GranularMarking",
    "Id",
    "Identity",
    "IdentityClassOv",
    "Indicator",
    "IndicatorTypeOv",
    "KillChainPhase",
    "MarkingDefinition",
    "PatternTypeOv",
    "common",
    "indicator",
    "open_vocab",
]



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .policy import Policy
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Authorization(UncheckedBaseModel):
    """
    The Authorization Result object provides details about the authorization outcome and associated policies related to activity.
    """

    decision: typing.Optional[str] = pydantic.Field(default=None)
    """
    Authorization Result/outcome, e.g. allowed, denied.
    """

    policy: typing.Optional[Policy] = pydantic.Field(default=None)
    """
    Details about the Identity/Access management policies that are applicable.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .assets_service_now_dataset import AssetsServiceNowDataset
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing
import pydantic


class AssetsServiceNowMock(UncheckedBaseModel):
    """
    Configuration for a mocked ServiceNow as an Assets Provider
    """

    dataset: AssetsServiceNowDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .policy import Policy
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Assessment(UncheckedBaseModel):
    """
    The Assessment object describes a point-in-time assessment, check, or evaluation of a specific configuration or signal against an asset, entity, person, or otherwise. For example, this can encapsulate <code>os_signals</code> from CrowdStrike Falcon Zero Trust Assessments, or account for <code>Datastore</code> configurations from Cyera, or capture details of Microsoft Intune configuration policies.
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    The category that the assessment is part of. For example: <code>Prevention</code> or <code>Windows 10</code>.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the assessment criteria, or a description of the specific configuration or signal the assessment is targeting.
    """

    meets_criteria: bool = pydantic.Field()
    """
    Determines whether the assessment against the specific configuration or signal meets the assessments criteria. For example, if the assessment checks if a <code>Datastore</code> is encrypted or not, having encryption would be evaluated as <code>true</code>.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the configuration or signal being assessed. For example: <code>Kernel Mode Code Integrity (KMCI)</code> or <code>publicAccessibilityState</code>.
    """

    policy: typing.Optional[Policy] = pydantic.Field(default=None)
    """
    The details of any policy associated with an assessment.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the configuration or signal being assessed. For example: the <code>signal_id</code>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class BuiltinRoles(str, enum.Enum):
    ROOT_ADMINISTRATOR = "root_administrator"
    ADMINISTRATOR = "administrator"
    VIEWER = "viewer"
    MEMBER = "member"
    _UNKNOWN = "__BUILTINROLES_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "BuiltinRoles":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        root_administrator: typing.Callable[[], T_Result],
        administrator: typing.Callable[[], T_Result],
        viewer: typing.Callable[[], T_Result],
        member: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is BuiltinRoles.ROOT_ADMINISTRATOR:
            return root_administrator()
        if self is BuiltinRoles.ADMINISTRATOR:
            return administrator()
        if self is BuiltinRoles.VIEWER:
            return viewer()
        if self is BuiltinRoles.MEMBER:
            return member()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class CreateBridgeRequest(UncheckedBaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique short name for this Bridge (lowercase [a-z0-9_-], can be used in URLs). Also used for case insensitive duplicate name detection and default sort order. Defaults to BridgeGroupId if both name and fullname are not specified.
    """

    fullname: typing.Optional[str] = pydantic.Field(default=None)
    """
    Human friendly display name for this Bridge, will auto-generate 'name' field (if 'name' is not specified). Defaults to the same value as the 'name' field if not specified.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the resources included in the bridge and permissions granted on those resources. Includes details of when to use this bridge along with the intended personas.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Bridge selection labels
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .agent import Agent
import pydantic
from .group import Group
from ...base.types.hostname import Hostname
from ...base.types.ip_address import IpAddress
from .user import User
from .key_value_object import KeyValueObject
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class ResourceDetails(UncheckedBaseModel):
    """
    The Resource Details object describes details about resources that were affected by the activity/event.
    """

    agent_list: typing.Optional[typing.List[Agent]] = pydantic.Field(default=None)
    """
    A list of <code>agent</code> objects associated with a device, endpoint, or resource.
    """

    cloud_partition: typing.Optional[str] = pydantic.Field(default=None)
    """
    The canonical cloud partition name to which the region is assigned (e.g. AWS Partitions: aws, aws-cn, aws-us-gov).
    """

    criticality: typing.Optional[str] = pydantic.Field(default=None)
    """
    The criticality of the resource as defined by the event source.
    """

    data: typing.Optional[typing.Optional[typing.Any]] = pydantic.Field(default=None)
    """
    Additional data describing the resource.
    """

    group: typing.Optional[Group] = pydantic.Field(default=None)
    """
    The name of the related resource group.
    """

    hostname: typing.Optional[Hostname] = pydantic.Field(default=None)
    """
    The fully qualified name of the resource.
    """

    ip: typing.Optional[IpAddress] = pydantic.Field(default=None)
    """
    The IP address of the resource, in either IPv4 or IPv6 format.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of labels associated to the resource.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the resource.
    """

    namespace: typing.Optional[str] = pydantic.Field(default=None)
    """
    The namespace is useful when similar entities exist that you need to keep separate.
    """

    owner: typing.Optional[User] = pydantic.Field(default=None)
    """
    The identity of the service or user account that owns the resource.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    The cloud region of the resource.
    """

    tags: typing.Optional[typing.List[KeyValueObject]] = pydantic.Field(default=None)
    """
    The list of tags; <code>{key:value}</code> pairs associated to the resource.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The resource type as defined by the event source.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the resource.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The version of the resource. For example <code>1.2.3</code>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from ...base.types.timestamp import Timestamp
import pydantic
import datetime as dt
from .user import User
from .product import Product
from .file_confidentiality_id import FileConfidentialityId
from .file_drive_type_id import FileDriveTypeId
from .encryption_details import EncryptionDetails
from .fingerprint import Fingerprint
from ...base.types.file_name import FileName
from .digital_signature import DigitalSignature
from .key_value_object import KeyValueObject
from .file_type_id import FileTypeId
from .url import Url
from .object import Object
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class File(UncheckedBaseModel):
    """
    The File object represents the metadata associated with a file stored in a computer system. It encompasses information about the file itself, including its attributes, properties, and organizational details.
    """

    accessed_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the file was last accessed.
    """

    accessed_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the file was last accessed.
    """

    accessor: typing.Optional[User] = pydantic.Field(default=None)
    """
    The name of the user who last accessed the object.
    """

    accessor_app: typing.Optional[Product] = pydantic.Field(default=None)
    """
    The app which mediated access to the file. This could be separate from the source app, for example a case where a user exports a report from a web app. The source app would be their browser and the accessor app the web app.
    """

    attributes: typing.Optional[int] = pydantic.Field(default=None)
    """
    The bitmask value that represents the file attributes.
    """

    company_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the company that published the file. For example: <code>Microsoft Corporation</code>.
    """

    confidentiality: typing.Optional[str] = pydantic.Field(default=None)
    """
    The file content confidentiality, normalized to the confidentiality_id value. In the case of 'Other', it is defined by the event source.
    """

    confidentiality_id: typing.Optional[FileConfidentialityId] = pydantic.Field(
        default=None
    )
    """
    The normalized identifier of the file content confidentiality indicator.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the file was created.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the file was created.
    """

    creator: typing.Optional[User] = pydantic.Field(default=None)
    """
    The user that created the file.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the file, as returned by file system. For example: the description as returned by the Unix file command or the Windows file type.
    """

    drive_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The drive type, normalized to the caption of the <code>drive_type_id</code> value. In the case of <code>Other</code>, it is defined by the source.
    """

    drive_type_id: typing.Optional[FileDriveTypeId] = pydantic.Field(default=None)
    """
    Identifies the type of a disk drive, i.e. fixed, removable, etc.
    """

    encryption_details: typing.Optional[EncryptionDetails] = pydantic.Field(
        default=None
    )
    """
    The encryption details of the file. Should be populated if the file is encrypted.
    """

    ext: typing.Optional[str] = pydantic.Field(default=None)
    """
    The extension of the file, excluding the leading dot. For example: <code>exe</code> from <code>svchost.exe</code>, or <code>gz</code> from <code>export.tar.gz</code>.
    """

    hashes: typing.Optional[typing.List[Fingerprint]] = pydantic.Field(default=None)
    """
    An array of hash attributes.
    """

    internal_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the file as identified within the file itself. This contrasts with the name by which the file is known on disk. Where available, the internal name is widely used by security practitioners and detection content because the on-disk file name is not reliable. On the Windows OS, most PE files contain a <a href="https://learn.microsoft.com/en-us/windows/win32/menurc/versioninfo-resource">VERSIONINFO</a> resource from which the internal name can be obtained. On macOS, binaries can optionally embed a copy of the application's Info.plist file which in turn contains the name of the executable.
    """

    is_deleted: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if the file was deleted from the filesystem.
    """

    is_encrypted: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if the file is encrypted.
    """

    is_public: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if the file is publicly accessible. For example in an object's public access in AWS S3
    """

    is_system: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The indication of whether the object is part of the operating system.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Labels associated with the object, such as security or sensitivity labels created by a scanning app.
    """

    mime_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Multipurpose Internet Mail Extensions (MIME) type of the file, if applicable.
    """

    modified_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the file was last modified.
    """

    modified_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the file was last modified.
    """

    modifier: typing.Optional[User] = pydantic.Field(default=None)
    """
    The user that last modified the file.
    """

    name: FileName = pydantic.Field()
    """
    The name of the file. For example: <code>svchost.exe</code>
    """

    owner: typing.Optional[User] = pydantic.Field(default=None)
    """
    The user that owns the file/object.
    """

    parent_folder: typing.Optional[str] = pydantic.Field(default=None)
    """
    The parent folder in which the file resides. For example: <code>c:\windows\system32</code>
    """

    path: typing.Optional[str] = pydantic.Field(default=None)
    """
    The full path to the file. For example: <code>c:\windows\system32\svchost.exe</code>.
    """

    product: typing.Optional[Product] = pydantic.Field(default=None)
    """
    The product that created or installed the file.
    """

    security_descriptor: typing.Optional[str] = pydantic.Field(default=None)
    """
    The object security descriptor.
    """

    signature: typing.Optional[DigitalSignature] = pydantic.Field(default=None)
    """
    The digital signature of the file.
    """

    size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The size of data, in bytes.
    """

    storage_class: typing.Optional[str] = pydantic.Field(default=None)
    """
    The storage class of the file. For example in AWS S3: <code>STANDARD, STANDARD_IA, GLACIER</code>.
    """

    tags: typing.Optional[typing.List[KeyValueObject]] = pydantic.Field(default=None)
    """
    The list of tags; <code>{key:value}</code> pairs associated to the file.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The file type.
    """

    type_id: FileTypeId = pydantic.Field()
    """
    The file type ID.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the file as defined by the storage system, such the file system file ID.
    """

    url: typing.Optional[Url] = pydantic.Field(default=None)
    """
    The URL of the file, when applicable.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The file version. For example: <code>8.0.7601.17514</code>.
    """

    xattributes: typing.Optional[Object] = pydantic.Field(default=None)
    """
    An unordered collection of zero or more name/value pairs where each pair represents a file or folder extended attribute.</p>For example: Windows alternate data stream attributes (ADS stream name, ADS size, etc.), user-defined or application-defined attributes, ACL, owner, primary group, etc. Examples from DCS: </p><ul><li><strong>ads_name</strong></li><li><strong>ads_size</strong></li><li><strong>dacl</strong></li><li><strong>owner</strong></li><li><strong>primary_group</strong></li><li><strong>link_name</strong> - name of the link associated to the file.</li><li><strong>hard_link_count</strong> - the number of links that are associated to the file.</li></ul>
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import datetime as dt
import pydantic
import typing
from .timeseries_result import TimeseriesResult
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class GetStatusTimeseriesResult(UncheckedBaseModel):
    """
    Status timeseries object
    """

    start_time: dt.datetime = pydantic.Field()
    """
    start time
    """

    end_time: dt.datetime = pydantic.Field()
    """
    end time
    """

    interval: str = pydantic.Field()
    """
    interval duration
    """

    series: typing.List[TimeseriesResult]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .options import Options
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class MemberOptions(UncheckedBaseModel):
    ttl: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional member time-to-live duration. After a member expires, system requires a change password to re-enable member. Minimum 1 day, Maximum 1 year, Default 180 days.
    """

    options: typing.Optional[typing.List[Options]] = pydantic.Field(default=None)
    """
    Options: "expired" will force change password on first logon.
    """

    token_ttl: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional token time-to-live duration. Tokens are created for this member with this duration as their TTL. Minimum 10 miniutes, Maximum 1 week, Defaults 1 hour.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from ...account_base.types.account_id import AccountId
import pydantic
from ...organization_base.types.environment import Environment
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class RoleAccounts(UncheckedBaseModel):
    ids: typing.List[AccountId] = pydantic.Field()
    """
    List of account ids that this role definition grants access to. Use "\*" to grant access to all account ids.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of account labels this role definition grants access to. If both labels and environments are specified both must pass
    """

    environments: typing.Optional[typing.List[Environment]] = pydantic.Field(
        default=None
    )
    """
    Account environments this role definition grants access to. If both labels and environments are specified both must pass
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .google_service_account_credential import GoogleServiceAccountCredential
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class SinkGoogleSecurityOperations(UncheckedBaseModel):
    """
    Configuration for Google Security Operations (formerly Google Chronicle) as a Sink Provider.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/google-security-operations-sink-setup)
    """

    credential: GoogleServiceAccountCredential = pydantic.Field()
    """
    Credentials used for accessing the Google SecOps instance.
    """

    customer_id: str = pydantic.Field()
    """
    The customer ID of the Google SecOps instance
    """

    project_id: str = pydantic.Field()
    """
    The project ID of the Google SecOps instance.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    The region of the Google SecOps instance. Usually `us` or `eu`.
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The base URL for the Google SecOps API.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .timespan import Timespan
import pydantic
from ...base.types.timestamp import Timestamp
import datetime as dt
from .advisory_install_state_id import AdvisoryInstallStateId
from .os import Os
from .product import Product
from .cve import Cve
from .cwe import Cwe
from ...base.types.url_string import UrlString
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Advisory(UncheckedBaseModel):
    """
    The Advisory object represents publicly disclosed cybersecurity vulnerabilities defined in a Security advisory. e.g. <code> Microsoft KB Article</code>, <code>Apple Security Advisory</code>, or a <code>GitHub Security Advisory (GHSA)</code>
    """

    avg_timespan: typing.Optional[Timespan] = pydantic.Field(default=None)
    """
    The average time to patch.
    """

    bulletin: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Advisory bulletin identifier.
    """

    classification: typing.Optional[str] = pydantic.Field(default=None)
    """
    The vendors classification of the Advisory.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the Advisory record was created.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the Advisory record was created.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    A brief description of the Advisory Record.
    """

    install_state: typing.Optional[str] = pydantic.Field(default=None)
    """
    The install state of the Advisory.
    """

    install_state_id: typing.Optional[AdvisoryInstallStateId] = pydantic.Field(
        default=None
    )
    """
    The normalized install state ID of the Advisory.
    """

    is_superseded: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The Advisory has been replaced by another.
    """

    modified_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the Advisory record was last updated.
    """

    modified_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the Advisory record was last updated.
    """

    os: typing.Optional[Os] = pydantic.Field(default=None)
    """
    The operating system the Advisory applies to.
    """

    product: typing.Optional[Product] = pydantic.Field(default=None)
    """
    The product where the vulnerability was discovered.
    """

    references: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of reference URLs with additional information about the vulnerabilities disclosed in the Advisory.
    """

    related_cves: typing.Optional[typing.List[Cve]] = pydantic.Field(default=None)
    """
    A list of Common Vulnerabilities and Exposures <a target='_blank' href='https://cve.mitre.org/'>(CVE)</a> identifiers related to the vulnerabilities disclosed in the Advisory.
    """

    related_cwes: typing.Optional[typing.List[Cwe]] = pydantic.Field(default=None)
    """
    A list of Common Weakness Enumeration <a target='_blank' href='https://cwe.mitre.org/'>(CWE)</a> identifiers related to the vulnerabilities disclosed in the Advisory.
    """

    size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The size in bytes for the Advisory. Usually populated for a KB Article patch.
    """

    src_url: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The Advisory link from the source vendor.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    A title or a brief phrase summarizing the Advisory.
    """

    uid: str = pydantic.Field()
    """
    The unique identifier assigned to the advisory or disclosed vulnerability, e.g, <code>GHSA-5mrr-rgp6-x4gr</code>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from ...base.types.email_address import EmailAddress
import pydantic
from .location import Location
from .domain_contact_type_id import DomainContactTypeId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class DomainContact(UncheckedBaseModel):
    """
    The contact information related to a domain registration, e.g., registrant, administrator, abuse, billing, or technical contact.
    """

    email_addr: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The user's primary email address.
    """

    location: typing.Optional[Location] = pydantic.Field(default=None)
    """
    Location details for the contract such as the city, state/province, country, etc.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The individual or organization name for the contact.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The number associated with the phone.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Domain Contact type, normalized to the caption of the <code>type_id</code> value. In the case of 'Other', it is defined by the source
    """

    type_id: DomainContactTypeId = pydantic.Field()
    """
    The normalized domain contact type ID.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the contact information, typically provided in WHOIS information.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...common.types.base import Base
from .integration_point_id import IntegrationPointId
import typing
import pydantic
from ...capabilities_base.types.category_id import CategoryId
from .integration_environments import IntegrationEnvironments
from .mapping_chain_template import MappingChainTemplate
from .additional_mapping_template import AdditionalMappingTemplate
from ...operation_schedules.types.operation_schedule import OperationSchedule
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class IntegrationPoint(Base):
    """
    Enables creation, editing and deletion of Integrations.
    """

    id: IntegrationPointId
    fullname: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of integration point, will be shown to end-users in the Connect UI.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional description of the Integration Point. Will not be displayed to end-users of Connect UI.
    """

    connector: CategoryId = pydantic.Field()
    """
    Connector to use for the Integration Point.
    """

    environments: IntegrationEnvironments = pydantic.Field()
    """
    Selects providers to use for account environments.
    """

    mappings: typing.Optional[typing.List[MappingChainTemplate]] = pydantic.Field(
        default=None
    )
    """
    A list of mapping chains to apply to integrations using this integration point. Each mapping chain is a list of mappings to apply to the integration in the order they should be applied. Mappings are applied by operation ID. If an integration is created that declares its own mappings for an operation, they will override this list of mappings. Leave this empty to use the default default mappings.
    """

    additional_mappings: typing.Optional[typing.List[AdditionalMappingTemplate]] = (
        pydantic.Field(default=None)
    )
    """
    Additional data mappings for integrations added to this integration point. This allows for custom data to be mapped to the custom_fields portion of the response.
    """

    scheduled_operations: typing.Optional[typing.List[OperationSchedule]] = (
        pydantic.Field(default=None)
    )
    """
    A list of operations that are scheduled to run for this integration point.
    Whenever an integration is created, these operations will automatically
    be scheduled to run based on the schedule defined for the operation.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from ...common.types.id import Id
import typing
from ...engine.types.object import Object
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class Agent(UncheckedBaseModel):
    """
    An agent is a human that can be notified when certain conditions are met
    """

    uid: Id
    full_name: str
    email_addr: typing.Optional[str] = None
    phone_number: typing.Optional[str] = None
    time_zone: typing.Optional[str] = None
    job_title: typing.Optional[str] = None
    unmapped: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The attributes that are not mapped to the comment schema. The names and values of those attributes are specific to the provider.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class OperationInput(UncheckedBaseModel):
    filters: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Limit the number of results returned by the operation. If not specified, the operation will return all results.
    """

    limit: typing.Optional[int] = pydantic.Field(default=None)
    """
    Limit query results by these filters.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .integration_web_hook_id import IntegrationWebHookId
import pydantic
import typing
import datetime as dt
from ...engine.types.object import Object
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class IntegrationWebHook(UncheckedBaseModel):
    """
    Integration webhook
    """

    id: IntegrationWebHookId
    creator: str = pydantic.Field()
    """
    The user who created the webHook.
    """

    enabled: bool = pydantic.Field()
    """
    Status of the WebHook.
    """

    events: typing.List[str] = pydantic.Field()
    """
    List of events that trigger the WebHook.
    """

    name: str = pydantic.Field()
    """
    A name for the WebHook.
    """

    value: str = pydantic.Field()
    """
    The content of the webHook, this can be a URL or a specific code, it depends on the provider.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    The date the WebHook was created.
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The date the WebHook was last updated.
    """

    unmapped: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The attributes that are not mapped to the WebHook schema. The names and values of those attributes are specific to the provider.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .cis_control import CisControl
import pydantic
from .kb_article import KbArticle
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Remediation(UncheckedBaseModel):
    """
    The Remediation object describes the recommended remediation steps to address identified issue(s).
    """

    cis_controls: typing.Optional[typing.List[CisControl]] = pydantic.Field(
        default=None
    )
    """
    An array of Center for Internet Security (CIS) Controls that can be optionally mapped to provide additional remediation details.
    """

    desc: str = pydantic.Field()
    """
    The description of the remediation strategy.
    """

    kb_article_list: typing.Optional[typing.List[KbArticle]] = pydantic.Field(
        default=None
    )
    """
    A list of KB articles or patches related to an endpoint. A KB Article contains metadata that describes the patch or an update.
    """

    kb_articles: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The KB article/s related to the entity. A KB Article contains metadata that describes the patch or an update.
    """

    references: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of supporting URL/s, references that help describe the remediation strategy.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .vendor_attributes_severity_id import VendorAttributesSeverityId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class VendorAttributes(UncheckedBaseModel):
    """
    The Vendor Attributes object can be used to represent values of attributes populated by the Vendor/Finding Provider. It can help distinguish between the vendor-prodvided values and consumer-updated values, of key attributes like <code>severity_id</code>.<br>The original finding producer should not populate this object. It should be populated by consuming systems that support data mutability.
    """

    severity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The finding severity, as reported by the Vendor (Finding Provider). The value should be normalized to the caption of the <code>severity_id</code> value. In the case of 'Other', it is defined by the source.
    """

    severity_id: typing.Optional[VendorAttributesSeverityId] = pydantic.Field(
        default=None
    )
    """
    The finding severity ID, as reported by the Vendor (Finding Provider).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .mitigation import Mitigation
import pydantic
from .sub_technique import SubTechnique
from .tactic import Tactic
from .technique import Technique
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Attack(UncheckedBaseModel):
    """
    The MITRE ATT&CK® & ATLAS™ object describes the tactic, technique, sub-technique & mitigation associated to an attack.
    """

    mitigation: typing.Optional[Mitigation] = pydantic.Field(default=None)
    """
    The Mitigation object describes the MITRE ATT&CK® or ATLAS™ Mitigation ID and/or name that is associated to an attack.
    """

    sub_technique: typing.Optional[SubTechnique] = pydantic.Field(default=None)
    """
    The Sub-technique object describes the MITRE ATT&CK® or ATLAS™ Sub-technique ID and/or name associated to an attack.
    """

    tactic: typing.Optional[Tactic] = pydantic.Field(default=None)
    """
    The Tactic object describes the MITRE ATT&CK® or ATLAS™ Tactic ID and/or name that is associated to an attack.
    """

    tactics: typing.Optional[typing.List[Tactic]] = pydantic.Field(default=None)
    """
    The Tactic object describes the tactic ID and/or tactic name that are associated with the attack technique, as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK® Matrix</a>.
    """

    technique: typing.Optional[Technique] = pydantic.Field(default=None)
    """
    The Technique object describes the MITRE ATT&CK® or ATLAS™ Technique ID and/or name associated to an attack.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ATT&CK® or ATLAS™ Matrix version.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...engine.types.api_response import ApiResponse
import typing
import pydantic
from ...ticketing.types.priority import Priority
from ...organization_webhook_base.types.webhook_event import WebhookEvent
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class TicketingWebhookResponse(ApiResponse):
    ticket_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the ticket in the ticketing system.
    """

    entity_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    Date pertiment to webhook operation e.g. if its TicketCreated would be creation date. This comes directly from the ticketing system without any date formatting changes.
    """

    priority: typing.Optional[Priority] = pydantic.Field(default=None)
    """
    The priority of the ticket.
    """

    short_description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Short Description of the ticket in the ticketing system.
    """

    event_type: typing.Optional[WebhookEvent] = pydantic.Field(default=None)
    """
    The type of event that was sent by the ticketing system.
    """

    raw_payload: str = pydantic.Field()
    """
    The Raw JSON that was sent by the ticketing system for this webhook event.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .process_entity import ProcessEntity
import pydantic
from .container import Container
from ...base.types.timestamp import Timestamp
import datetime as dt
from .environment_variable import EnvironmentVariable
from .file import File
from .group import Group
from .process_integrity_id import ProcessIntegrityId
from ...base.types.file_path import FilePath
from ...base.types.process_name import ProcessName
from .object import Object
from .session import Session
from .user import User
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Process(UncheckedBaseModel):
    """
    The Process object describes a running instance of a launched program.
    """

    ancestry: typing.Optional[typing.List[ProcessEntity]] = pydantic.Field(default=None)
    """
    An array of Process Entities describing the extended parentage of this process object. Direct parent information sould be expressed through the <code>parent_process</code> attribute. The first array element is the direct parent of this process object. Subsequent list elements go up the process parentage hierarchy. That is, the array is sorted from newest to oldest process. It is recommended to only populate this field for the top-level process object.
    """

    auid: typing.Optional[int] = pydantic.Field(default=None)
    """
    The audit user assigned at login by the audit subsystem.
    """

    cmd_line: typing.Optional[str] = pydantic.Field(default=None)
    """
    The full command line used to launch an application, service, process, or job. For example: <code>ssh user@10.0.0.10</code>. If the command line is unavailable or missing, the empty string <code>''</code> is to be used.
    """

    container: typing.Optional[Container] = pydantic.Field(default=None)
    """
    The information describing an instance of a container. A container is a prepackaged, portable system image that runs isolated on an existing system using a container runtime like containerd.
    """

    cpid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique process identifier that can be assigned deterministically by multiple system data producers.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the process was created/started.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the process was created/started.
    """

    egid: typing.Optional[int] = pydantic.Field(default=None)
    """
    The effective group under which this process is running.
    """

    environment_variables: typing.Optional[typing.List[EnvironmentVariable]] = (
        pydantic.Field(default=None)
    )
    """
    Environment variables associated with the process.
    """

    euid: typing.Optional[int] = pydantic.Field(default=None)
    """
    The effective user under which this process is running.
    """

    file: typing.Optional[File] = pydantic.Field(default=None)
    """
    The process file object.
    """

    group: typing.Optional[Group] = pydantic.Field(default=None)
    """
    The group under which this process is running.
    """

    integrity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The process integrity level, normalized to the caption of the integrity_id value. In the case of 'Other', it is defined by the event source (Windows only).
    """

    integrity_id: typing.Optional[ProcessIntegrityId] = pydantic.Field(default=None)
    """
    The normalized identifier of the process integrity level (Windows only).
    """

    lineage: typing.Optional[typing.List[FilePath]] = pydantic.Field(default=None)
    """
    The lineage of the process, represented by a list of paths for each ancestor process. For example: <code>['/usr/sbin/sshd', '/usr/bin/bash', '/usr/bin/whoami']</code>.
    """

    loaded_modules: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of loaded module names.
    """

    name: typing.Optional[ProcessName] = pydantic.Field(default=None)
    """
    The friendly name of the process, for example: <code>Notepad++</code>.
    """

    namespace_pid: typing.Optional[int] = pydantic.Field(default=None)
    """
    If running under a process namespace (such as in a container), the process identifier within that process namespace.
    """

    parent_process: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The parent process of this process object. It is recommended to only populate this field for the top-level process object, to prevent deep nesting. Additional ancestry information can be supplied in the <code>ancestry</code> attribute.
    """

    path: typing.Optional[str] = pydantic.Field(default=None)
    """
    The process file path.
    """

    pid: typing.Optional[int] = pydantic.Field(default=None)
    """
    The process identifier, as reported by the operating system. Process ID (PID) is a number used by the operating system to uniquely identify an active process.
    """

    sandbox: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the containment jail (i.e., sandbox). For example, hardened_ps, high_security_ps, oracle_ps, netsvcs_ps, or default_ps.
    """

    session: typing.Optional[Session] = pydantic.Field(default=None)
    """
    The user session under which this process is running.
    """

    terminated_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the process was terminated.
    """

    terminated_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the process was terminated.
    """

    tid: typing.Optional[int] = pydantic.Field(default=None)
    """
    The Identifier of the thread associated with the event, as returned by the operating system.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique identifier for this process assigned by the producer (tool). Facilitates correlation of a process event with other events for that process.
    """

    user: typing.Optional[User] = pydantic.Field(default=None)
    """
    The user under which this process is running.
    """

    working_directory: typing.Optional[str] = pydantic.Field(default=None)
    """
    The working directory of a process.
    """

    xattributes: typing.Optional[Object] = pydantic.Field(default=None)
    """
    An unordered collection of zero or more name/value pairs that represent a process extended attribute.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...common.types.base import Base
from .member_id import MemberId
from .state import State
import datetime as dt
import pydantic
import typing
from ...role_base.types.role_name import RoleName
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class Member(Base):
    id: MemberId
    state: State
    last_logon: dt.datetime = pydantic.Field()
    """
    Last logon time
    """

    fullname: str = pydantic.Field()
    """
    User's full display name.
    """

    nickname: typing.Optional[str] = pydantic.Field(default=None)
    """
    User's nickname
    """

    picture: typing.Optional[str] = pydantic.Field(default=None)
    """
    Url of user's picture
    """

    ttl: str
    token_ttl: str
    expires: dt.datetime
    pin_expires: dt.datetime
    role_binding: typing.List[RoleName] = pydantic.Field()
    """
    Roles granted to this member. Tokens inherit this access.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .operation_incremental_pull_mode import OperationIncrementalPullMode
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class OperationIncrementalPullConfig(UncheckedBaseModel):
    """
    Controls how data is fetched from the source API on each execution.
    """

    mode: OperationIncrementalPullMode = pydantic.Field()
    """
    Fetch strategy: incremental (only changed data) or full_snapshot (all data every time).
    """

    time_filter_field: typing.Optional[str] = pydantic.Field(default=None)
    """
    Custom field to use for time-based filtering in incremental mode.
    Overrides the operation's default time filter field.
    Examples: "modified_time" or "finding.last_seen_time".
    The filter will use this field with "[gt]" (greater than) operator.
    First run will use `initial_backfill_start_time` as the filter value, and
    subsequent runs use the last execution time.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .patch_organization_response import PatchOrganizationResponse
from .update_organization_request import UpdateOrganizationRequest
from .update_organization_response import UpdateOrganizationResponse

__all__ = [
    "PatchOrganizationResponse",
    "UpdateOrganizationRequest",
    "UpdateOrganizationResponse",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClarotyApiurl(str, enum.Enum):
    CLAROTY_API_URL_US = "https://api.claroty.com"
    CLAROTY_API_URL_EU = "https://eu.api.claroty.com"
    CLAROTY_API_URL_CA = "https://ca.api.claroty.com"
    CLAROTY_API_URL_AU = "https://au.api.claroty.com"
    CLAROTY_API_URL_DEMO = "https://demo-api.claroty.com"
    _UNKNOWN = "__CLAROTYAPIURL_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ClarotyApiurl":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        claroty_api_url_us: typing.Callable[[], T_Result],
        claroty_api_url_eu: typing.Callable[[], T_Result],
        claroty_api_url_ca: typing.Callable[[], T_Result],
        claroty_api_url_au: typing.Callable[[], T_Result],
        claroty_api_url_demo: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ClarotyApiurl.CLAROTY_API_URL_US:
            return claroty_api_url_us()
        if self is ClarotyApiurl.CLAROTY_API_URL_EU:
            return claroty_api_url_eu()
        if self is ClarotyApiurl.CLAROTY_API_URL_CA:
            return claroty_api_url_ca()
        if self is ClarotyApiurl.CLAROTY_API_URL_AU:
            return claroty_api_url_au()
        if self is ClarotyApiurl.CLAROTY_API_URL_DEMO:
            return claroty_api_url_demo()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .http_header import HttpHeader
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class HttpResponse(UncheckedBaseModel):
    """
    The HTTP Response object contains detailed information about the response sent from a web server to the requester. It encompasses attributes and metadata that describe the response status, headers, body content, and other relevant information.
    """

    code: int = pydantic.Field()
    """
    The Hypertext Transfer Protocol (HTTP) status code returned from the web server to the client. For example, 200.
    """

    content_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The request header that identifies the original <a target='_blank' href='https://www.iana.org/assignments/media-types/media-types.xhtml'>media type </a> of the resource (prior to any content encoding applied for sending).
    """

    http_headers: typing.Optional[typing.List[HttpHeader]] = pydantic.Field(
        default=None
    )
    """
    Additional HTTP headers of an HTTP request or response.
    """

    latency: typing.Optional[int] = pydantic.Field(default=None)
    """
    The HTTP response latency measured in milliseconds.
    """

    length: typing.Optional[int] = pydantic.Field(default=None)
    """
    The HTTP response length, in number of bytes.
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the event/finding, as defined by the source.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The response status. For example: A successful HTTP status of 'OK' which corresponds to a code of 200.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from ...ocsf.v_1_3_0.securityfinding.classes.types.severity_id import SeverityId
import typing
from .vulnerability_severity_filter_value import VulnerabilitySeverityFilterValue
import pydantic
from .vulnerability_state_filter_value import VulnerabilityStateFilterValue
from .provider_specific_finding_state import ProviderSpecificFindingState
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class UpdateFindingRequest(UncheckedBaseModel):
    severity_id: SeverityId
    severity: typing.Optional[VulnerabilitySeverityFilterValue] = pydantic.Field(
        default=None
    )
    """
    severity of the finding
    """

    state: typing.Optional[VulnerabilityStateFilterValue] = pydantic.Field(default=None)
    """
    state of the finding
    """

    unmapped: typing.Optional[ProviderSpecificFindingState] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .resources import Resources
from .adhoc_role import AdhocRole
from .role_accounts import RoleAccounts
from .role_id import RoleId
from .role_integrations import RoleIntegrations
from .role_name import RoleName
from .role_organizations import RoleOrganizations

__all__ = [
    "AdhocRole",
    "Resources",
    "RoleAccounts",
    "RoleId",
    "RoleIntegrations",
    "RoleName",
    "RoleOrganizations",
]



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .product import Product
from ...base.types.timestamp import Timestamp
import datetime as dt
from ...base.types.url_string import UrlString
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class TransformationInfo(UncheckedBaseModel):
    """
    The transformation_info object represents the mapping or transformation used.
    """

    lang: typing.Optional[str] = pydantic.Field(default=None)
    """
    The transformation language used to transform the data.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the transformation or mapping.
    """

    product: typing.Optional[Product] = pydantic.Field(default=None)
    """
    The product or instance used to make the transformation
    """

    time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    Time of the transformation.
    """

    time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Time of the transformation.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the mapping or transformation.
    """

    url_string: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The Uniform Resource Locator String where the mapping or transformation exists.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ...base.types.url_string import UrlString
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class SubTechnique(UncheckedBaseModel):
    """
    The MITRE ATT&CK® Sub Technique object describes the sub technique ID and/or name associated to an attack, as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK® Matrix</a>.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the attack sub technique, as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK® Matrix</a>. For example: <code>Scanning IP Blocks</code>.
    """

    src_url: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The versioned permalink of the attack sub technique, as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK® Matrix</a>. For example: <code>https://attack.mitre.org/versions/v14/techniques/T1595/001/</code>.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the attack sub technique, as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK® Matrix</a>. For example: <code>T1595.001</code>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class PingOneAuthUrl(str, enum.Enum):
    PINGONE_AUTH_URL_US = "https://auth.pingone.com"
    PINGONE_AUTH_URL_CA = "https://auth.pingone.ca"
    PINGONE_AUTH_URL_EU = "https://auth.pingone.eu"
    PINGONE_AUTH_URL_AU = "https://auth.pingone.com.au"
    PINGONE_AUTH_URL_SG = "https://auth.pingone.sg"
    PINGONE_AUTH_URL_ASIA = "https://auth.pingone.asia"
    _UNKNOWN = "__PINGONEAUTHURL_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "PingOneAuthUrl":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        pingone_auth_url_us: typing.Callable[[], T_Result],
        pingone_auth_url_ca: typing.Callable[[], T_Result],
        pingone_auth_url_eu: typing.Callable[[], T_Result],
        pingone_auth_url_au: typing.Callable[[], T_Result],
        pingone_auth_url_sg: typing.Callable[[], T_Result],
        pingone_auth_url_asia: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is PingOneAuthUrl.PINGONE_AUTH_URL_US:
            return pingone_auth_url_us()
        if self is PingOneAuthUrl.PINGONE_AUTH_URL_CA:
            return pingone_auth_url_ca()
        if self is PingOneAuthUrl.PINGONE_AUTH_URL_EU:
            return pingone_auth_url_eu()
        if self is PingOneAuthUrl.PINGONE_AUTH_URL_AU:
            return pingone_auth_url_au()
        if self is PingOneAuthUrl.PINGONE_AUTH_URL_SG:
            return pingone_auth_url_sg()
        if self is PingOneAuthUrl.PINGONE_AUTH_URL_ASIA:
            return pingone_auth_url_asia()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Permissions(str, enum.Enum):
    ADMINISTRATOR = "administrator"
    VIEWER = "viewer"
    MEMBER = "member"
    ACCOUNT_MANAGER = "account-manager"
    CONNECT_UI = "connect-ui"
    TOKEN_ISSUER = "token-issuer"
    MCP_INTEGRATIONS_USE_ONLY = "mcp-integrations-use-only"
    """
    Permission set that provides the minimum level of access necessary for MCP use. Gives read access to accounts and integrations and the ability to use any Connector API.
    """

    MCP_MANAGEMENT = "mcp-management"
    """
    Permission set that provides the minimum level of access necessary for MCP management use. Can create and update integrations but cannot use any Connector API.
    """

    _UNKNOWN = "__PERMISSIONS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "Permissions":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        administrator: typing.Callable[[], T_Result],
        viewer: typing.Callable[[], T_Result],
        member: typing.Callable[[], T_Result],
        account_manager: typing.Callable[[], T_Result],
        connect_ui: typing.Callable[[], T_Result],
        token_issuer: typing.Callable[[], T_Result],
        mcp_integrations_use_only: typing.Callable[[], T_Result],
        mcp_management: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is Permissions.ADMINISTRATOR:
            return administrator()
        if self is Permissions.VIEWER:
            return viewer()
        if self is Permissions.MEMBER:
            return member()
        if self is Permissions.ACCOUNT_MANAGER:
            return account_manager()
        if self is Permissions.CONNECT_UI:
            return connect_ui()
        if self is Permissions.TOKEN_ISSUER:
            return token_issuer()
        if self is Permissions.MCP_INTEGRATIONS_USE_ONLY:
            return mcp_integrations_use_only()
        if self is Permissions.MCP_MANAGEMENT:
            return mcp_management()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .file import File
import pydantic
from .fingerprint import Fingerprint
from .long_string import LongString
from .script_type_id import ScriptTypeId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Script(UncheckedBaseModel):
    """
    The Script object describes a script or command that can be executed by a shell, script engine, or interpreter. Examples include Bash, JavsScript, PowerShell, Python, VBScript, etc. Note that the term <em>script</em> here denotes not only a script contained within a file but also a script or command typed interactively by a user, supplied on the command line, or provided by some other file-less mechanism.
    """

    file: typing.Optional[File] = pydantic.Field(default=None)
    """
    Present if this script is associated with a file. Not present in the case of a file-less script.
    """

    hashes: typing.Optional[typing.List[Fingerprint]] = pydantic.Field(default=None)
    """
    An array of the script's cryptographic hashes. Note that these hashes are calculated on the script in its original encoding, and not on the normalized UTF-8 encoding found in the <code>script_content</code> attribute.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the script or macro, independent of the containing file, used for tracking, auditing, and security analysis.
    """

    parent_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    This attribute relates a sub-script to a parent script having the matching <code>uid</code> attribute. In the case of PowerShell, sub-script execution can be identified by matching the activity correlation ID of the raw ETW events provided by the OS.
    """

    script_content: LongString = pydantic.Field()
    """
    The script content, normalized to UTF-8 encoding irrespective of its original encoding. When emitting this attribute, it may be appropriate to truncate large scripts. When consuming this attribute, large scripts should be anticipated.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The script type, normalized to the caption of the <code>type_id</code> value. In the case of 'Other', it is defined by the event source.
    """

    type_id: ScriptTypeId = pydantic.Field()
    """
    The normalized script type ID.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Some script engines assign a unique ID to each individual execution of a given script. This attribute captures that unique ID. In the case of PowerShell, the unique ID corresponds to the <code>ScriptBlockId</code> in the raw ETW events provided by the OS.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...common.types.base import Base
from ...token_base.types.token_id import TokenId
import pydantic
import typing
from ...common.types.id import Id
from ...token_base.types.token_owner_type import TokenOwnerType
import datetime as dt
from ...token_base.types.token_type import TokenType
from ...token_base.types.token_pair import TokenPair
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class RefreshToken(Base):
    id: TokenId
    fullname: str = pydantic.Field()
    """
    Human friendly display name for this account.
    """

    member_id: typing.Optional[Id] = pydantic.Field(default=None)
    """
    Member Id
    """

    owner_id: Id = pydantic.Field()
    """
    ID of the entity that owns this token
    """

    owner_type: TokenOwnerType = pydantic.Field()
    """
    Type of the entity that owns this token
    """

    expires: dt.datetime = pydantic.Field()
    """
    Time when this token expires and can no longer be used again.
    """

    token_ttl: str = pydantic.Field()
    """
    Token time-to-live
    """

    token_type: TokenType = pydantic.Field()
    """
    Type of token created
    """

    primary: TokenPair = pydantic.Field()
    """
    Primary running access and refresh tokens
    """

    secondary: typing.Optional[TokenPair] = pydantic.Field(default=None)
    """
    Temporary secondary TokenPair created after a RefreshToken operation
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .cvss_depth import CvssDepth
from .metric import Metric
from ...base.types.url_string import UrlString
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Cvss(UncheckedBaseModel):
    """
    The Common Vulnerability Scoring System (<a target='_blank' href='https://www.first.org/cvss/'>CVSS</a>) object provides a way to capture the principal characteristics of a vulnerability and produce a numerical score reflecting its severity.
    """

    base_score: float = pydantic.Field()
    """
    The CVSS base score. For example: <code>9.1</code>.
    """

    depth: typing.Optional[CvssDepth] = pydantic.Field(default=None)
    """
    The CVSS depth represents a depth of the equation used to calculate CVSS score.
    """

    metrics: typing.Optional[typing.List[Metric]] = pydantic.Field(default=None)
    """
    The Common Vulnerability Scoring System metrics. This attribute contains information on the CVE's impact. If the CVE has been analyzed, this attribute will contain any CVSSv2 or CVSSv3 information associated with the vulnerability. For example: <code>{ {"Access Vector", "Network"}, {"Access Complexity", "Low"}, ...}</code>.
    """

    overall_score: typing.Optional[float] = pydantic.Field(default=None)
    """
    The CVSS overall score, impacted by base, temporal, and environmental metrics. For example: <code>9.1</code>.
    """

    severity: typing.Optional[str] = pydantic.Field(default=None)
    """
    <p>The Common Vulnerability Scoring System (CVSS) Qualitative Severity Rating. A textual representation of the numeric score.</p><strong>CVSS v2.0</strong><ul><li>Low (0.0 – 3.9)</li><li>Medium (4.0 – 6.9)</li><li>High (7.0 – 10.0)</li></ul></p><strong>CVSS v3.0</strong><ul><li>None (0.0)</li><li>Low (0.1 - 3.9)</li><li>Medium (4.0 - 6.9)</li><li>High (7.0 - 8.9)</li><li>Critical (9.0 - 10.0)</li></ul>
    """

    src_url: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The source URL for the CVSS score. For example: <code>https://nvd.nist.gov/vuln/detail/CVE-2021-44228</code>
    """

    vector_string: typing.Optional[str] = pydantic.Field(default=None)
    """
    The CVSS vector string is a text representation of a set of CVSS metrics. It is commonly used to record or transfer CVSS metric information in a concise form. For example: <code>3.1/AV:L/AC:L/PR:L/UI:N/S:U/C:H/I:N/A:H</code>.
    """

    vendor_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The vendor that provided the CVSS score. For example: <code>NVD, REDHAT</code> etc.
    """

    version: str = pydantic.Field()
    """
    The CVSS version. For example: <code>3.1</code>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class FirewallRule(UncheckedBaseModel):
    """
    The Firewall Rule object represents a specific rule within a firewall policy or event. It contains information about a rule's configuration, properties, and associated actions that define how network traffic is handled by the firewall.
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    The rule category.
    """

    condition: typing.Optional[str] = pydantic.Field(default=None)
    """
    The rule trigger condition for the rule. For example: SQL_INJECTION.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the rule that generated the event.
    """

    duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    The rule response time duration, usually used for challenge completion time.
    """

    match_details: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The data in a request that rule matched. For example: '["10","and","1"]'.
    """

    match_location: typing.Optional[str] = pydantic.Field(default=None)
    """
    The location of the matched data in the source which resulted in the triggered firewall rule. For example: HEADER.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the rule that generated the event.
    """

    rate_limit: typing.Optional[int] = pydantic.Field(default=None)
    """
    The rate limit for a rate-based rule.
    """

    sensitivity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The sensitivity of the firewall rule in the matched event. For example: HIGH.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The rule type.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the rule that generated the event.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The rule version. For example: <code>1.1</code>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    EmailAddress,
    FileName,
    Hash,
    Hostname,
    IpAddress,
    MacAddress,
    Port,
    ProcessName,
    ResourceUid,
    Subnet,
    Timestamp,
    UrlString,
    UserName,
)

__all__ = [
    "EmailAddress",
    "FileName",
    "Hash",
    "Hostname",
    "IpAddress",
    "MacAddress",
    "Port",
    "ProcessName",
    "ResourceUid",
    "Subnet",
    "Timestamp",
    "UrlString",
    "UserName",
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    CreateOrganizationWebhookRequest,
    CreateOrganizationWebhookResponse,
    GetOrganizationWebhookResponse,
    ListOrganizationWebhooksResponse,
    OrganizationWebhook,
    OrganizationWebhookSecret,
    PatchOrganizationWebhookResponse,
    UpdateOrganizationWebhookRequest,
    UpdateOrganizationWebhookResponse,
)

__all__ = [
    "CreateOrganizationWebhookRequest",
    "CreateOrganizationWebhookResponse",
    "GetOrganizationWebhookResponse",
    "ListOrganizationWebhooksResponse",
    "OrganizationWebhook",
    "OrganizationWebhookSecret",
    "PatchOrganizationWebhookResponse",
    "UpdateOrganizationWebhookRequest",
    "UpdateOrganizationWebhookResponse",
]



# This file was auto-generated by Fern from our API Definition.

from ....v_1_3_0.dnsactivity.classes.types.dns_activity import (
    DnsActivity as ocsf_v_1_3_0_dnsactivity_classes_types_dns_activity_DnsActivity,
)

DnsActivity = ocsf_v_1_3_0_dnsactivity_classes_types_dns_activity_DnsActivity



# This file was auto-generated by Fern from our API Definition.

import typing
from .environment import SynqlyManagementEnvironment
import httpx
from .core.client_wrapper import SyncClientWrapper
from .accounts.client import AccountsClient
from .audit.client import AuditClient
from .auth.client import AuthClient
from .bridges.client import BridgesClient
from .capabilities.client import CapabilitiesClient
from .credentials.client import CredentialsClient
from .customs.client import CustomsClient
from .integration_points.client import IntegrationPointsClient
from .integrations.client import IntegrationsClient
from .mappings.client import MappingsClient
from .members.client import MembersClient
from .meta.client import MetaClient
from .operations.client import OperationsClient
from .organization_webhooks.client import OrganizationWebhooksClient
from .organization.client import OrganizationClient
from .permissionset.client import PermissionsetClient
from .roles.client import RolesClient
from .status.client import StatusClient
from .sub_orgs.client import SubOrgsClient
from .tokens.client import TokensClient
from .core.client_wrapper import AsyncClientWrapper
from .accounts.client import AsyncAccountsClient
from .audit.client import AsyncAuditClient
from .auth.client import AsyncAuthClient
from .bridges.client import AsyncBridgesClient
from .capabilities.client import AsyncCapabilitiesClient
from .credentials.client import AsyncCredentialsClient
from .customs.client import AsyncCustomsClient
from .integration_points.client import AsyncIntegrationPointsClient
from .integrations.client import AsyncIntegrationsClient
from .mappings.client import AsyncMappingsClient
from .members.client import AsyncMembersClient
from .meta.client import AsyncMetaClient
from .operations.client import AsyncOperationsClient
from .organization_webhooks.client import AsyncOrganizationWebhooksClient
from .organization.client import AsyncOrganizationClient
from .permissionset.client import AsyncPermissionsetClient
from .roles.client import AsyncRolesClient
from .status.client import AsyncStatusClient
from .sub_orgs.client import AsyncSubOrgsClient
from .tokens.client import AsyncTokensClient


class SynqlyManagement:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : SynqlyManagementEnvironment
        The environment to use for requests from the client. from .environment import SynqlyManagementEnvironment



        Defaults to SynqlyManagementEnvironment.SYNQLY



    token : typing.Union[str, typing.Callable[[], str]]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from synqly import SynqlyManagement

    client = SynqlyManagement(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: SynqlyManagementEnvironment = SynqlyManagementEnvironment.SYNQLY,
        token: typing.Union[str, typing.Callable[[], str]],
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else None
        )
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(
                timeout=_defaulted_timeout, follow_redirects=follow_redirects
            )
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.accounts = AccountsClient(client_wrapper=self._client_wrapper)
        self.audit = AuditClient(client_wrapper=self._client_wrapper)
        self.auth = AuthClient(client_wrapper=self._client_wrapper)
        self.bridges = BridgesClient(client_wrapper=self._client_wrapper)
        self.capabilities = CapabilitiesClient(client_wrapper=self._client_wrapper)
        self.credentials = CredentialsClient(client_wrapper=self._client_wrapper)
        self.customs = CustomsClient(client_wrapper=self._client_wrapper)
        self.integration_points = IntegrationPointsClient(
            client_wrapper=self._client_wrapper
        )
        self.integrations = IntegrationsClient(client_wrapper=self._client_wrapper)
        self.mappings = MappingsClient(client_wrapper=self._client_wrapper)
        self.members = MembersClient(client_wrapper=self._client_wrapper)
        self.meta = MetaClient(client_wrapper=self._client_wrapper)
        self.operations = OperationsClient(client_wrapper=self._client_wrapper)
        self.organization_webhooks = OrganizationWebhooksClient(
            client_wrapper=self._client_wrapper
        )
        self.organization = OrganizationClient(client_wrapper=self._client_wrapper)
        self.permissionset = PermissionsetClient(client_wrapper=self._client_wrapper)
        self.roles = RolesClient(client_wrapper=self._client_wrapper)
        self.status = StatusClient(client_wrapper=self._client_wrapper)
        self.sub_orgs = SubOrgsClient(client_wrapper=self._client_wrapper)
        self.tokens = TokensClient(client_wrapper=self._client_wrapper)


class AsyncSynqlyManagement:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : SynqlyManagementEnvironment
        The environment to use for requests from the client. from .environment import SynqlyManagementEnvironment



        Defaults to SynqlyManagementEnvironment.SYNQLY



    token : typing.Union[str, typing.Callable[[], str]]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from synqly import AsyncSynqlyManagement

    client = AsyncSynqlyManagement(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: SynqlyManagementEnvironment = SynqlyManagementEnvironment.SYNQLY,
        token: typing.Union[str, typing.Callable[[], str]],
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else None
        )
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(
                timeout=_defaulted_timeout, follow_redirects=follow_redirects
            )
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.accounts = AsyncAccountsClient(client_wrapper=self._client_wrapper)
        self.audit = AsyncAuditClient(client_wrapper=self._client_wrapper)
        self.auth = AsyncAuthClient(client_wrapper=self._client_wrapper)
        self.bridges = AsyncBridgesClient(client_wrapper=self._client_wrapper)
        self.capabilities = AsyncCapabilitiesClient(client_wrapper=self._client_wrapper)
        self.credentials = AsyncCredentialsClient(client_wrapper=self._client_wrapper)
        self.customs = AsyncCustomsClient(client_wrapper=self._client_wrapper)
        self.integration_points = AsyncIntegrationPointsClient(
            client_wrapper=self._client_wrapper
        )
        self.integrations = AsyncIntegrationsClient(client_wrapper=self._client_wrapper)
        self.mappings = AsyncMappingsClient(client_wrapper=self._client_wrapper)
        self.members = AsyncMembersClient(client_wrapper=self._client_wrapper)
        self.meta = AsyncMetaClient(client_wrapper=self._client_wrapper)
        self.operations = AsyncOperationsClient(client_wrapper=self._client_wrapper)
        self.organization_webhooks = AsyncOrganizationWebhooksClient(
            client_wrapper=self._client_wrapper
        )
        self.organization = AsyncOrganizationClient(client_wrapper=self._client_wrapper)
        self.permissionset = AsyncPermissionsetClient(
            client_wrapper=self._client_wrapper
        )
        self.roles = AsyncRolesClient(client_wrapper=self._client_wrapper)
        self.status = AsyncStatusClient(client_wrapper=self._client_wrapper)
        self.sub_orgs = AsyncSubOrgsClient(client_wrapper=self._client_wrapper)
        self.tokens = AsyncTokensClient(client_wrapper=self._client_wrapper)


def _get_base_url(
    *, base_url: typing.Optional[str] = None, environment: SynqlyManagementEnvironment
) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception(
            "Please pass in either base_url or environment to construct the client"
        )



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from ...common.types.id import Id
import pydantic
import datetime as dt
import typing
from .operation_trigger_type import OperationTriggerType
from ...operation_base.types.operation_status import OperationStatus
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class OperationExecutionEvent(UncheckedBaseModel):
    """
    A single execution of an operation, capturing start time, completion time,
    and metrics for that specific run.
    """

    execution_id: Id = pydantic.Field()
    """
    Unique identifier for this execution
    """

    operation_id: Id = pydantic.Field()
    """
    ID of the operation that was executed
    """

    started_at: dt.datetime = pydantic.Field()
    """
    When this execution started
    """

    completed_at: dt.datetime = pydantic.Field()
    """
    When this execution completed
    """

    account_id: Id = pydantic.Field()
    """
    Account ID containing the integration
    """

    integration_id: Id = pydantic.Field()
    """
    Integration ID used for the operation
    """

    integration_point_id: typing.Optional[Id] = pydantic.Field(default=None)
    """
    Integration-point ID if triggered by a schedule, omitted if manually triggered
    via the API
    """

    operation: str = pydantic.Field()
    """
    Name of the operation (e.g., "assets_query_devices")
    """

    trigger_type: OperationTriggerType = pydantic.Field()
    """
    How this execution was triggered
    """

    trigger_source: str = pydantic.Field()
    """
    Source of trigger - "manual" for API calls, integration-point ID for scheduled
    """

    status: OperationStatus = pydantic.Field()
    """
    Final status of the execution
    """

    records_processed: int = pydantic.Field()
    """
    Approximate number of records processed
    """

    cpu_time: int = pydantic.Field()
    """
    CPU time in microseconds
    """

    in_bytes: int = pydantic.Field()
    """
    Number of bytes sent to sink integration
    """

    duration: int = pydantic.Field()
    """
    Total execution duration in microseconds
    """

    initial_cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cursor value at execution start
    """

    final_cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cursor value at execution completion
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message if execution failed
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .device_hw_info_cpu_architecture_id import DeviceHwInfoCpuArchitectureId
from .display import Display
from .keyboard_info import KeyboardInfo
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class DeviceHwInfo(UncheckedBaseModel):
    """
    The Device Hardware Information object contains details and specifications of the physical components that make up a device. This information provides an overview of the hardware capabilities, configuration, and characteristics of the device.
    """

    bios_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    The BIOS date. For example: <code>03/31/16</code>.
    """

    bios_manufacturer: typing.Optional[str] = pydantic.Field(default=None)
    """
    The BIOS manufacturer. For example: <code>LENOVO</code>.
    """

    bios_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The BIOS UUID.
    """

    bios_ver: typing.Optional[str] = pydantic.Field(default=None)
    """
    The BIOS version. For example: <code>LENOVO G5ETA2WW (2.62)</code>.
    """

    chassis: typing.Optional[str] = pydantic.Field(default=None)
    """
    The chassis type describes the system enclosure or physical form factor. Such as the following examples for Windows <a target='_blank' href='https://docs.microsoft.com/en-us/windows/win32/cimwin32prov/win32-systemenclosure'>Windows Chassis Types</a>
    """

    cpu_architecture: typing.Optional[str] = pydantic.Field(default=None)
    """
    The CPU architecture, normalized to the caption of the <code>cpu_architecture_id</code> value. In the case of <code>Other</code>, it is defined by the source.
    """

    cpu_architecture_id: typing.Optional[DeviceHwInfoCpuArchitectureId] = (
        pydantic.Field(default=None)
    )
    """
    The normalized identifier of the CPU architecture.
    """

    cpu_bits: typing.Optional[int] = pydantic.Field(default=None)
    """
    The cpu architecture, the number of bits used for addressing in memory. For example: <code>32</code> or <code>64</code>.
    """

    cpu_cores: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of processor cores in all installed processors. For Example: <code>42</code>.
    """

    cpu_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of physical processors on a system. For example: <code>1</code>.
    """

    cpu_speed: typing.Optional[int] = pydantic.Field(default=None)
    """
    The speed of the processor in Mhz. For Example: <code>4200</code>.
    """

    cpu_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The processor type. For example: <code>x86 Family 6 Model 37 Stepping 5</code>.
    """

    desktop_display: typing.Optional[Display] = pydantic.Field(default=None)
    """
    The desktop display affiliated with the event
    """

    keyboard_info: typing.Optional[KeyboardInfo] = pydantic.Field(default=None)
    """
    The keyboard detailed information.
    """

    ram_size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total amount of installed RAM, in Megabytes. For example: <code>2048</code>.
    """

    serial_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The device manufacturer serial number.
    """

    uuid_: typing.Optional[str] = pydantic.Field(alias="uuid", default=None)
    """
    The device manufacturer assigned universally unique hardware identifier. For example: The BIOS System UUID or the Apple IOPlatformUUID.
    """

    vendor_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The device manufacturer.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow
